<manifest lang="yaml">
name: WebContainer Compiler
description: A webassembly based node environment for compiling frontend projects.
type: web-app
entry_point: https://oeway.github.io/hypha-webcontainer-compiler/
version: "1.0.0"
inject_auth_params: true
extra_http_headers:
    Access-Control-Allow-Origin: "*"
    Access-Control-Allow-Methods: "GET, POST, PUT, DELETE, OPTIONS"
    Access-Control-Allow-Headers: "Content-Type, Authorization, X-Requested-With"
startup_config:
    wait_for_service: webcontainer-compiler-worker
</manifest>